# Copyright 2012 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class FakeDriver:
    def __init__(self, first_arg=True):
        self.first_arg = first_arg


class FakeDriver2:
    def __init__(self, first_arg):
        self.first_arg = first_arg


class FakeDriver3:
    def __init__(self):
        raise ImportError("ImportError occurs in __init__")
